import aiohttp
import asyncio
import logging
from pprint import pformat

import config


class UIWSTest:

    HOST = "http://127.0.0.1:%d" % config.get_settings()['web_interface']['port']
    #HOST = "https://127.0.0.1:%d" % config.get_settings()['web_interface']['port_https']
    LOGIN_PATH = "/login"
    WS_HOST = "ws://127.0.0.1:%d/ui_ws" % config.get_settings()['web_interface']['port']
    #WS_HOST = "ws://127.0.0.1:%d/ui_ws" % config.get_settings()['web_interface']['port_https']
    HEADERS = {}
    TIMEOUT = 2
    LOOP_TIME = 0.1
    REQ_PER_TEST = 128
    CORRECT_LOGIN = "vladimir@3dprinteros.com"
    CORRECT_PASSWORD = "pofigpofig"

    def __init__(self):
        self.logger = logging.getLogger(__class__.__name__)
        self.stop_flag = False
        self.is_operational = False
        self.ws = None
        self.host = self.HOST
        self.headers = self.HEADERS
        self.timeout_obj = aiohttp.ClientTimeout(total=self.TIMEOUT)
        try:
            self.loop = asyncio.get_event_loop()
        except RuntimeError:
            self.logger.info("Staring event loop...")
            self.loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.loop)

    def run(self):
        self.loop.run_until_complete(self.test())

    def close(self):
        self.stop_flag = True

    def create_login_form(self, login, password):
        form = aiohttp.FormData()
        form.add_field('login', login)
        form.add_field('password', password)
        return form

    def parse_message(self, message):
        self.logger.debug("WS recv message: %s" % message)

    async def test(self):
        self.logger.info("Connecting to %s..." % self.host)

        try:
            async with aiohttp.ClientSession(timeout=self.timeout_obj) as session:
                #  # Test login fail
                #  async with session.post(self.host + self.LOGIN_PATH,\
                #                               data=self.create_login_form("foo", "bar"),\
                #                               timeout=self.timeout_obj) as resp:
                #      if resp.status == 200:
                #          text = await resp.text()
                #          self.logger.info("Failed login response: %s" % text)
                #  # Test login success
                #  async with session.post(self.host + self.LOGIN_PATH,\
                #                               data=self.create_login_form(self.CORRECT_LOGIN, self.CORRECT_PASSWORD),\
                #                               timeout=self.timeout_obj) as resp:
                #       if resp.status == 200:
                #          text = await resp.text()
                #          self.logger.info("Correct login response: %s" % text)
                await self.ws_communication_loop(session)
        except Exception as e:
            self.logger.exception("Exception: " + str(e) )
            self.stop_flag = True

    async def ws_communication_loop(self, session):
        self.logger.info("Connecting to websocket %s" % self.WS_HOST)
        counter = 0
        async with session.ws_connect(self.WS_HOST) as ws:
            while not self.stop_flag and counter < self.REQ_PER_TEST:
                msg = await ws.receive()
                self.logger.info('Received on WS:' + str(msg.data))
                if self.stop_flag:
                    self.logger.debug('Exiting WS loop due to stop flag...')
                    break
                elif msg.type == aiohttp.WSMsgType.TEXT:
                    self.parse_message(msg.data)
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    self.logger.warning('Exiting WS loop due to error message...')
                    break
                elif msg.type == aiohttp.WSMsgType.CLOSED:
                    self.logger.warning('Exiting WS loop due to server socket close...')
                    break
                #await asyncio.sleep(self.LOOP_TIME)
                await ws.send_json("%s" % counter)
                counter += 1
        self.logger.debug('Websocket disconnected')


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    try:
        test = UIWSTest()
        test.run()
    except KeyboardInterrupt:
        test.close()
    finally:
        test.close()
